@@ -21,7 +21,7 @@
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
  *
  *
- *  $Id: l2cap.c,v 1.41 2005/02/01 18:51:42 holtmann Exp $
+ *  $Id: l2cap.c,v 1.45 2005/05/15 20:28:18 holtmann Exp $
  */
 
 #include <stdio.h>
@@ -50,9 +50,11 @@ typedef struct {
 static handle_info handle_table[HANDLE_TABLE_SIZE];
 
 typedef struct {
+	uint16_t handle;
 	uint16_t cid;
 	uint16_t psm;
 	uint16_t num;
+	uint8_t mode;
 } cid_info;
 #define CID_TABLE_SIZE 20
 
@@ -86,7 +88,7 @@ static struct frame *get_frame(uint16_t handle)
 	return add_handle(handle);
 }
 
-static void add_cid(int in, uint16_t cid, uint16_t psm)
+static void add_cid(int in, uint16_t handle, uint16_t cid, uint16_t psm)
 {
 	register cid_info *table = cid_table[in];
 	register int i, pos = -1;
@@ -100,9 +102,11 @@ static void add_cid(int in, uint16_t cid, uint16_t psm)
 	}
 
 	if (pos >= 0) {
-		table[pos].cid = cid;
-		table[pos].psm = psm;
-		table[pos].num = num;
+		table[pos].handle = handle;
+		table[pos].cid    = cid;
+		table[pos].psm    = psm;
+		table[pos].num    = num;
+		table[pos].mode   = 0;
 	}
 }
 
@@ -122,14 +126,32 @@ static void del_cid(int in, uint16_t dcid, uint16_t scid)
 	for (t = 0; t < 2; t++) {
 		for (i = 0; i < CID_TABLE_SIZE; i++)
 			if (cid_table[t][i].cid == cid[t]) {
-				cid_table[t][i].cid = 0;
-				cid_table[t][i].psm = 0;
-				cid_table[t][i].num = 0;
+				cid_table[t][i].handle = 0;
+				cid_table[t][i].cid    = 0;
+				cid_table[t][i].psm    = 0;
+				cid_table[t][i].num    = 0;
+				cid_table[t][i].mode   = 0;
 				break;
 			}
 	}
 }
 
+static void del_handle(uint16_t handle)
+{
+	register int t, i;
+
+	for (t = 0; t < 2; t++) {
+		for (i = 0; i < CID_TABLE_SIZE; i++)
+			if (cid_table[t][i].handle == handle) {
+				cid_table[t][i].handle = 0;
+				cid_table[t][i].cid    = 0;
+				cid_table[t][i].psm    = 0;
+				cid_table[t][i].num    = 0;
+				cid_table[t][i].mode   = 0;
+				break;
+			}
+	}
+}
 static uint16_t get_psm(int in, uint16_t cid)
 {
 	register cid_info *table = cid_table[in];
@@ -152,6 +174,27 @@ static uint16_t get_num(int in, uint16_t cid)
 	return 0;
 }
 
+static void set_mode(int in, uint16_t cid, uint8_t mode)
+{
+	register cid_info *table = cid_table[in];
+	register int i;
+
+	for (i = 0; i < CID_TABLE_SIZE; i++)
+		if (table[i].cid == cid)
+			table[i].mode = mode;
+}
+
+static uint8_t get_mode(int in, uint16_t cid)
+{
+	register cid_info *table = cid_table[in];
+	register int i;
+
+	for (i = 0; i < CID_TABLE_SIZE; i++)
+		if (table[i].cid == cid)
+			return table[i].mode;
+	return 0;
+}
+
 static uint32_t get_val(uint8_t *ptr, uint8_t len)
 {
 	switch (len) {
@@ -266,6 +309,38 @@ static char *mode2str(uint8_t mode)
 	}
 }
 
+static char *sar2str(uint8_t sar)
+{
+	switch (sar) {
+	case 0x00:
+		return "Unsegmented";
+	case 0x01:
+		return "Start";
+	case 0x02:
+		return "End";
+	case 0x03:
+		return "Continuation";
+	default:
+		return "Bad SAR";
+
+	}
+}
+
+static char *supervisory2str(uint8_t supervisory)
+{
+	switch (supervisory) {
+	case 0x00:
+		return "Receiver Ready (RR)";
+	case 0x01:
+		return "Reject (REJ)";
+	case 0x02:
+	case 0x03:
+		return "Reserved Supervisory";
+	default:
+		return "Bad Supervisory";
+	}
+}
+
 static inline void command_rej(int level, struct frame *frm)
 {
 	l2cap_cmd_rej *h = frm->ptr;
@@ -287,10 +362,8 @@ static inline void command_rej(int level, struct frame *frm)
 		break;
 	}
 
-	if (parser.flags & DUMP_VERBOSE) {
-		p_indent(level + 1, frm);
-		printf("%s\n", reason2str(reason));
-	}
+	p_indent(level + 1, frm);
+	printf("%s\n", reason2str(reason));
 }
 
 static inline void conn_req(int level, struct frame *frm)
@@ -299,7 +372,7 @@ static inline void conn_req(int level, struct frame *frm)
 	uint16_t psm = btohs(h->psm);
 	uint16_t scid = btohs(h->scid);
 
-	add_cid(frm->in, scid, psm);
+	add_cid(frm->in, frm->handle, scid, psm);
 
 	if (p_filter(FILT_L2CAP))
 		return;
@@ -319,7 +392,7 @@ static inline void conn_rsp(int level, struct frame *frm)
 	switch (h->result) {
 	case L2CAP_CR_SUCCESS:
 		if ((psm = get_psm(!frm->in, scid)))
-			add_cid(frm->in, dcid, psm);
+			add_cid(frm->in, frm->handle, dcid, psm);
 		break;
 
 	case L2CAP_CR_PEND:
@@ -336,22 +409,21 @@ static inline void conn_rsp(int level, struct frame *frm)
 	printf("Connect rsp: dcid 0x%4.4x scid 0x%4.4x result %d status %d\n",
 		dcid, scid, result, status);
 
-	if (parser.flags & DUMP_VERBOSE) {
-		p_indent(level + 1, frm);
-		printf("%s", connresult2str(result));
+	p_indent(level + 1, frm);
+	printf("%s", connresult2str(result));
 
-		if (result == 0x0001)
-			printf(" - %s\n", status2str(status));
-		else
-			printf("\n");
-	}
+	if (result == 0x0001)
+		printf(" - %s\n", status2str(status));
+	else
+		printf("\n");
 }
 
-static void conf_opt(int level, void *ptr, int len)
+static void conf_opt(int level, void *ptr, int len, int in, uint16_t cid)
 {
 	p_indent(level, 0);
 	while (len > 0) {
 		l2cap_conf_opt *h = ptr;
+		uint8_t mode;
 
 		ptr += L2CAP_CONF_OPT_SIZE + h->len;
 		len -= L2CAP_CONF_OPT_SIZE + h->len;
@@ -380,8 +452,11 @@ static void conf_opt(int level, void *ptr, int len)
 
 		case L2CAP_CONF_RFC:
 			printf("Mode");
-			if (h->len > 0)
-				printf(" 0x02%x (%s)", *h->val, mode2str(*h->val));
+			if (h->len > 0) {
+				mode = *h->val;
+				set_mode(in, cid, mode);
+				printf(" 0x%02x (%s)", mode, mode2str(mode));
+			}
 			break;
 
 		default:
@@ -400,21 +475,23 @@ static void conf_opt(int level, void *ptr, int len)
 static inline void conf_req(int level, l2cap_cmd_hdr *cmd, struct frame *frm)
 {
 	l2cap_conf_req *h = frm->ptr;
+	uint16_t dcid = btohs(h->dcid);
 	int clen = btohs(cmd->len) - L2CAP_CONF_REQ_SIZE;
 
 	if (p_filter(FILT_L2CAP))
 		return;
 
 	printf("Config req: dcid 0x%4.4x flags 0x%2.2x clen %d\n",
-			btohs(h->dcid), btohs(h->flags), clen);
+			dcid, btohs(h->flags), clen);
 
-	if (clen)
-		conf_opt(level + 1, h->data, clen);
+	if (clen > 0)
+		conf_opt(level + 1, h->data, clen, frm->in, dcid);
 }
 
 static inline void conf_rsp(int level, l2cap_cmd_hdr *cmd, struct frame *frm)
 {
 	l2cap_conf_rsp *h = frm->ptr;
+	uint16_t scid = btohs(h->scid);
 	uint16_t result = btohs(h->result);
 	int clen = btohs(cmd->len) - L2CAP_CONF_RSP_SIZE;
 
@@ -422,15 +499,18 @@ static inline void conf_rsp(int level, l2cap_cmd_hdr *cmd, struct frame *frm)
 		return;
 
 	printf("Config rsp: scid 0x%4.4x flags 0x%2.2x result %d clen %d\n",
-			btohs(h->scid), btohs(h->flags), result, clen);
+			scid, btohs(h->flags), result, clen);
 
-	if (parser.flags & DUMP_VERBOSE) {
+	if (clen > 0) {
+		if (!result) {
+			p_indent(level + 1, frm);
+			printf("%s\n", confresult2str(result));
+		}
+		conf_opt(level + 1, h->data, clen, frm->in, scid);
+	} else {
 		p_indent(level + 1, frm);
 		printf("%s\n", confresult2str(result));
 	}
-
-	if (clen)
-		conf_opt(level + 1, h->data, clen);
 }
 
 static inline void disconn_req(int level, struct frame *frm)
@@ -479,6 +559,8 @@ static inline void echo_rsp(int level, l2cap_cmd_hdr *cmd, struct frame *frm)
 
 static void info_opt(int level, int type, void *ptr, int len)
 {
+	uint32_t mask;
+
 	p_indent(level, 0);
 
 	switch (type) {
@@ -486,7 +568,22 @@ static void info_opt(int level, int type, void *ptr, int len)
 		printf("Connectionless MTU %d\n", get_val(ptr, len));
 		break;
 	case 0x0002:
-		printf("Extended feature mask 0x%4.4x\n", get_val(ptr, len));
+		mask = get_val(ptr, len);
+		printf("Extended feature mask 0x%4.4x\n", mask);
+		if (parser.flags & DUMP_VERBOSE) {
+			if (mask & 0x01) {
+				p_indent(level + 1, 0);
+				printf("Flow control mode\n");
+			}
+			if (mask & 0x02) {
+				p_indent(level + 1, 0);
+				printf("Retransmission mode\n");
+			}
+			if (mask & 0x04) {
+				p_indent(level + 1, 0);
+				printf("Bi-directional QoS\n");
+			}
+		}
 		break;
 	default:
 		printf("Unknown (len %d)\n", len);
@@ -516,13 +613,12 @@ static inline void info_rsp(int level, l2cap_cmd_hdr *cmd, struct frame *frm)
 
 	printf("Info rsp: type %d result %d\n", type, result);
 
-	if (parser.flags & DUMP_VERBOSE) {
+	if (ilen > 0) {
+		info_opt(level + 1, type, h->data, ilen);
+	} else {
 		p_indent(level + 1, frm);
 		printf("%s\n", inforesult2str(result));
 	}
-
-	if (ilen)
-		info_opt(level + 1, type, h->data, ilen);
 }
 
 static void l2cap_parse(int level, struct frame *frm)
@@ -610,7 +706,8 @@ static void l2cap_parse(int level, struct frame *frm)
 		if (p_filter(FILT_L2CAP))
 			return;
 
-		psm = btohs(bt_get_unaligned((uint16_t*)frm->ptr));
+		psm = btohs(bt_get_unaligned((uint16_t *) frm->ptr));
+		frm->ptr += 2;
 		frm->len -= 2;
 
 		p_indent(level, frm);
@@ -619,17 +716,50 @@ static void l2cap_parse(int level, struct frame *frm)
 	} else {
 		/* Connection oriented channel */
 
+		uint8_t mode = get_mode(!frm->in, cid);
 		uint16_t psm = get_psm(!frm->in, cid);
+		uint16_t ctrl = 0, fcs = 0;
 		uint32_t proto;
 
 		frm->cid = cid;
 		frm->num = get_num(!frm->in, cid);
 
+		if (mode > 0) {
+			ctrl = btohs(bt_get_unaligned((uint16_t *) frm->ptr));
+			frm->ptr += 2;
+			frm->len -= 4;
+			fcs = btohs(bt_get_unaligned((uint16_t *) (frm->ptr + frm->len)));
+		}
+
 		if (!p_filter(FILT_L2CAP)) {
 			p_indent(level, frm);
-			printf("L2CAP(d): cid 0x%4.4x len %d [psm %d]\n",
-				cid, dlen, psm);
+			printf("L2CAP(d): cid 0x%4.4x len %d", cid, dlen);
+			if (mode > 0)
+				printf(" ctrl 0x%4.4x fcs 0x%4.4x", ctrl, fcs);
+			printf(" [psm %d]\n", psm);
 			level++;
+			if (mode > 0) {
+				p_indent(level, frm);
+				printf("%s:", ctrl & 0x01 ? "S-frame" : "I-frame");
+				if (ctrl & 0x01) {
+					printf(" %s", supervisory2str((ctrl & 0x0c) >> 2));
+				} else {
+					uint8_t sar = (ctrl & 0xc000) >> 14;
+					printf(" %s", sar2str(sar));
+					if (sar == 1) {
+						uint16_t len;
+						len = btohs(bt_get_unaligned((uint16_t *) frm->ptr));
+						frm->ptr += 2;
+						frm->len -= 2;
+						printf(" (len %d)", len);
+					}
+					printf(" TxSeq %d", (ctrl & 0x7e) >> 1);
+				}
+				printf(" ReqSeq %d", (ctrl & 0x3f00) >> 8);
+				if (ctrl & 0x80)
+					printf(" Retransmission Disable");
+				printf("\n");
+			}
 		}
 
 		switch (psm) {
@@ -721,7 +851,8 @@ void l2cap_dump(int level, struct frame *frm)
 			return;
 		}
 
-		if (fr->data) free(fr->data);
+		if (fr->data)
+			free(fr->data);
 
 		if (!(fr->data = malloc(dlen + L2CAP_HDR_SIZE))) {
 			perror("Can't allocate L2CAP reassembly buffer");
@@ -736,6 +867,7 @@ void l2cap_dump(int level, struct frame *frm)
 		fr->handle  = frm->handle;
 		fr->cid     = frm->cid;
 		fr->num     = frm->num;
+		fr->dlci    = frm->dlci;
 		fr->channel = frm->channel;
 	} else {
 		if (!(fr = get_frame(frm->handle))) {
@@ -769,3 +901,8 @@ void l2cap_dump(int level, struct frame *frm)
 		}
 	}
 }
+
+void l2cap_clear(uint16_t handle)
+{
+	del_handle(handle);
+}
